;-------------------------------------------------------------------------;
.include "c64_pal.inc"
.include "graphics.inc"
.include "snes.inc"
.include "snes_decompress.inc"
.include "snes_zvars.inc"
;-------------------------------------------------------------------------;
.import clear_vram
.export DoFlex
;-------------------------------------------------------------------------;


;-------------------------------------------------------------------------;
RAM_SCROLLTEXT = 0d00h
GFX_BUFFER = 7e8000h
RAM_CHARSET = 0e00h
;-------------------------------------------------------------------------;


;-------------------------------------------------------------------------;
gfx_offset = m0
row_counter = m1
storage1 = m2
storage2 = m3
;-------------------------------------------------------------------------;


;-------------------------------;-----------------------------------------;
BG1MAP = 07000h			; 7400h & 7800h will be used
BG3MAP = 00c00h			;
BG4MAP = 07c00h
;-------------------------------;-----------------------------------------;
BG3GFX = 1000h
BG4GFX = 2000h


;/////////////////////////////////////////////////////////////////////////;
.zeropage
;/////////////////////////////////////////////////////////////////////////;


;-------------------------------------------------------------------------;
bg1hofs:
	.res 1
char_line:
	.res 1
flex_column:
	.res 1
scroll_toff:
	.res 2
sine_offset:
	.res 2
text_roff:
	.res 2
;-------------------------------------------------------------------------;


;=========================================================================;
;      Code (c) 1993-94 -Pan-/ANTHROX   All code can be used at will!
;=========================================================================;                     

;/////////////////////////////////////////////////////////////////////////;
.code
;/////////////////////////////////////////////////////////////////////////;


	.a8
	.i16


;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::;
DoFlex:
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::;

	rep	#10h		; X,Y fixed -> 16 bit mode
	sep	#20h		; Accumulator ->  8 bit mode

	stz	REG_BGMODE	

	lda	#BG1MAP>>8|SC_64x32
				; BG0 Tile Address $7400 (2 screens in use)
				; when bit 1 is on it means to use 2
				; horizontal screens
				; you can see the other screen if you use
				; REG_BG1HOFS to scroll the screen
				; this is used because when scrolling the
				; screen the LEFT MOST character on the screen
				; shows up on the right most side of the screen
				; WE DON'T WANT TO SEE THAT CHARACTER!
	sta	REG_BG1SC
	sta	REG_BG2SC

	lda	#BG3MAP>>8
	sta	REG_BG3SC
	lda	#BG4MAP>>8
	sta	REG_BG4SC

	stz	REG_BG12NBA	; BG0 Graphics data $0000
	lda	#BG3GFX>>12+BG4GFX>>8
	sta	REG_BG34NBA

	lda	#0ffh
	sta	REG_BG1VOFS
	stz	REG_BG1VOFS
	stz	REG_BG2VOFS
	stz	REG_BG2VOFS
	sta	REG_BG3VOFS
	stz	REG_BG3VOFS
	sta	REG_BG4VOFS
	stz	REG_BG4VOFS

	jsr	clear_vram

	DoDecompressDataVram gfx_flex_8x8Tiles, BG3GFX*2
	DoDecompressDataVram gfx_colorbarTiles, BG4GFX*2
	DoDecompressDataVram gfx_colorbarMap, BG4MAP*2
	DoCopyPalette BG1_PAL,  0, 2
	DoCopyPalette BG2_PAL, 32, 2
	DoCopyPalette BG3_PAL, 64, 4
	DoCopyPalette gfx_colorbarPal, 96, 4

        lda     #VMAIN_INCH
        sta     REG_VMAIN

        ldx     #BG3GFX
        stx     REG_VMADDL
	ldx	#0000h
				; from Anomie's Register Doc:
        lda     REG_VMDATALREAD	; << When either of these registers is read, the
:       lda     REG_VMDATALREAD	; appropriate byte from a word-sized buffer is
        sta     RAM_CHARSET,x	; returned. A word from VRAM is loaded into this
	inx			; buffer just before the VRAM address is incremented
	lda	REG_VMDATAHREAD	; The actual data read and the amount of the
	sta	RAM_CHARSET,x	; increment depend on the low 4 bits of $2115.
	inx			; The effect of this is that a 'dummy read' is
        cpx     #0400h		; required after setting $2116-7 before you start
        bne     :-		; getting the actual data.

	jsr	MakeTiles	; set up the screen
	jsr	ClearRam	; clear ram "V-Ram" buffer for flex routine

	stz	REG_DMAP0	; graphical glitches occur outside of the flex
	lda	#<REG_TM	; area which were not visible on the original demo
	sta	REG_BBAD0	; since the top and bottom of the screen were
	ldx	#LIST_TM	; all black
	stx	REG_A1T0L
	stz	REG_A1B0

	lda	#1
	sta	REG_HDMAEN

	lda	#^GFX_BUFFER	; a = graphics buffer bank
	ldx	#GFX_BUFFER	; x = graphics buffer address
	stx	memptr		; lda	[memptr] is equal to lda GFX_BUFFER
	sta	memptr+2

	lda	#0fh
	sta	REG_INIDISP	; turn the screen on
;-------------------------------------------------------------------------;
Waitloop:
;-------------------------------------------------------------------------;
	jsr	WaitVb		; wait for vertical blank
	jsr	Routines	; go do the scroll/flex routines
	bra	Waitloop	; constant loop

;===========================================================================
;                     Start of Vertical Blank Interrupt Routine
;===========================================================================

Routines:
				; start of General DMA graphics copy routine!
	stz	REG_DMAP3	; 0= 1 byte per register (not a word!)
	lda	#<REG_VMDATA
	sta	REG_BBAD3	; 21xx   this is 2118 (VRAM)
	ldx	#GFX_BUFFER
	stx	REG_A1T3L
	lda	#^GFX_BUFFER
	sta	REG_A1B3	; bank address of data in ram
	ldx	#0840h		; ** upped to 840h for the extra column **
	stx	REG_DAS3L	; # of bytes to be transferred
	stz	REG_VMAIN	; increase V-Ram address after writing to
				; REG_VMDATAL
	ldx	#0000h
	stx	REG_VMADDL	; address of VRAM to copy garphics in
	lda	#%1000		; turn on bit 4 (%1000=8) of G-DMA channel
	sta	REG_MDMAEN
	lda	#VMAIN_INCH	; increase V-Ram address after writing to
	sta	REG_VMAIN	; REG_VMDATAH

	jsr	Scroll		; go to scroll text routine
	jsr	Flex		; go to flex routine
	rts


;============================================================================
Scroll:
;============================================================================
	inc	bg1hofs
	lda	bg1hofs		; get current horizontal scroll offset
	sta	REG_BG1HOFS	; scroll the screen
	stz	REG_BG1HOFS	; clear other bits

	ina			; offset for shadow (sc2)
	sta	REG_BG2HOFS
	stz	REG_BG2HOFS

	cmp	#09h		; did it move 8 pixels?
	bcs	scrolltexts	; yeah, maybe even more!
	rts
;-------------------------------------------------------------------------;
scrolltexts:
;-------------------------------------------------------------------------;
	lda	#0ffh
	sta	bg1hofs		; reset the position to 0ffh

	ldx	#0000h
	ldy	scroll_toff	; get the scroll text offset
stiny:	iny			; increase it by 1 and store it
	sty	scroll_toff
;-------------------------------------------------------------------------;
copyscroll:
;-------------------------------------------------------------------------;
	lda	SCROLLTEXT,y	; read the scroll text
	beq	endscroll	; did it reach the end? if so go to endscroll
	sta	RAM_SCROLLTEXT,x; store the data into ram
	iny
	inx
	cpx	#0021h		; copy 33 characters into ram
	bne	copyscroll
;-------------------------------------------------------------------------;
	rts
;-------------------------------------------------------------------------;
endscroll:
;-------------------------------------------------------------------------;
	ldy	#0000h
	bra	stiny		; beginning, go back and re-copy


;===========================================================================;
Flex:
;===========================================================================;
	ldy	sine_offset	; read REAL sine offset
	sty	storage2	; store it to use in creation of all flexing
;-------------------------------; chars
Flex1:
;-------------------------------;
	ldy	storage2	; read the offset
	sty	storage1	; store it to use in creation of a single
				; flexing char
	ldx	text_roff	; read current text ram offset

	rep	#30h		; A,X,Y = 16 bit
				;
	lda	RAM_SCROLLTEXT,x; read scroll text char
	and	#003fh		; convert ASCII->C64 charset code
				; and mask out high byte
	asl a	;(*2)		; to take the current Char Value and find it's
	asl a	;(*4)		; graphic offset since the charset is 16 lines
	asl a	;(*8)		; high we need to multiply by 16
	asl a	;(*16)		; the easy way to do this is to use 4 ASLs
	sta	gfx_offset	; store it (removing this causes a brk in some emu
	tax			; put graphics offset in X register
;-------------------------------;-----------------------------------------;
Flexdraw:
;-------------------------------;-----------------------------------------;
	ldy	storage1	; read sine offset for single flexing char
				;
	rep	#30h		; A,X,Y = 16 bits
				;
	lda	SINE,y		; read sine data
	and	#00ffh		; leave out hi byte
	tay			; transfer A to Y
				;
	sep	#20h		; A = 8 bits
				;
	lda	RAM_CHARSET,x	; read the charset lying at RAM_CHARSET
	sta	[memptr],y	; store it in current Flex Char Column
				; this works much better than 
				; STA GFX_BUFFER,x since we can't re-write
				; rom to increase to the next column
				; we use zero page addressing
				; which we can modify as much as we want!
	inx			; get next graphic byte
	inc	storage1	; get next sine offset value
	inc	char_line	; 
	lda	char_line	; get # of graphics drawn
	cmp	#10h		; did we do 16 lines?
	bne	Flexdraw	; no! continue finishing the character
;-------------------------------;-----------------------------------------;
	stz	char_line	; all done, now to reset it to zero
	inc	flex_column	; go to the next column
	lda	flex_column	;
	cmp	#21h		; did we do 33 columns?
	beq	okgood		; if so then branch to okgood
;-------------------------------;-----------------------------------------;
	rep	#30h		; no, make A = 16

	lda	memptr		; read address stored in memptr
	clc
	adc	#0040h		; add 64 to the address. since each column
				; is 8 chars high and each char is 8 lines
				; high we multiply 8*8 to 64 (64 = $40)
				; thats how we get to the next flex column
	sta	memptr		; store next column address into memptr

	sep	#20h		; A = 8

	lda	storage2
	clc
	adc	#0feh		; 0feh is going backwards 2 (same as sbc #02h)
	sta	storage2	; increase sine offset before going to draw
				; the next character
				; the higher the number, the steeper the
				; sine angle will be
	inc	text_roff	; increase text ram offset
	bra	Flex1		; go to Flex1
;-------------------------------;-----------------------------------------;
	rts
;-------------------------------------------------------------------------;
okgood:
;-------------------------------------------------------------------------;
	ldx	#GFX_BUFFER	; put GFX_BUFFER address into memptr
	stx	memptr		; memptr+2 won't change since nothing
				; affected it. 
	stz	text_roff	; reset text ram offset back to $0
	stz	flex_column	; reset column counter back to $0
	inc	sine_offset	;
	inc	sine_offset	; increase REAL sine offset to get movement
				; 2 incs move the sine faster
	rts


;=========================================================================;
WaitVb:				; Vertical Blank Wait Routine
;=========================================================================;
	lda	REG_RDNMI	; is the number higher than #$7f? (#$80-$ff)
	bpl     WaitVb		; bpl tests bit #7 ($80) if this bit is set
				; it means the byte is negative (BMI, Branch
				; on Minus) BPL (Branch on Plus) if bit #7 is
				; set in REG_RDNMI it means that it is at the
				; start of V-Blank if not it will keep testing
				; REG_RDNMI until bit #7 is on (which would
	rts			; make it a negative (BMI)


;=========================================================================;
;       	     SETUP ROUTINES FOR PROGRAM
;=========================================================================;


;=========================================================================;
MakeTiles:
;=========================================================================;
	ldx	#BG1MAP
	stx	REG_VMADDL
	ldx	#0000h
	txy
	lda	#%100
;-------------------------------------------------------------------------;
clearscreen:
;---------------------------------------;---------------------------------;
	stz	REG_VMDATAL		; clear the text screen (with unused tile)
	sta	REG_VMDATAH		; ''
	inx				;
	cpx	#0400h*2		; clear 7400h & 7800h
	bne	clearscreen		;
;---------------------------------------;---------------------------------;
	;ldx	#7800h			; unneeded because tile tilemaps
	;stx	REG_VMADDL		; are contiguous just use 0800h instead
	;ldx	#0000h			; of 0400h
;clearscreen2:
	;stz	REG_VMDATAL		; clear second screen (unseen except for
	;sta	REG_VMDATAH		; right most side of monitor)
	;iny
	;cpy	#0400h
	;bne	clearscreen2

	ldx	#BG3MAP+2e0h
	stx	REG_VMADDL
:	lda	TEXT,y
	and	#3fh
	sta	REG_VMDATAL
	stz	REG_VMDATAH
	iny
	cpy	#TEXT_END-TEXT
	bne	:-

;===========================================================================;
;                          Flex Column Tile Draw Routine
;===========================================================================;


; the flex routine works easily with tiles because of the format tiles
; are stored in.    We read characters left to right, but the CPU
; reads it top to bottom.   the characters sets are stored in sequence
; we see ABCDEFG
; while the computer knows:
; A
; B
; C
; D
; E
; F
; G

; and since each letter is made up 8 bytes then the byte are in sequence
; also!

; this means we can make a column of characters in sequence and re-draw the
; graphics to make them look like they are bouncing (DYCP) or totally bend
; (FLEX) 


	ldx	#BG1MAP+140h	; set the V-ram address to start drawing
	stx	REG_VMADDL	; columns
;-------------------------------------------------------------------------;
DrawChar:
;-------------------------------------------------------------------------;
	ldx	#0000h
;-------------------------------------------------------------------------;
;drawchar:
;-------------------------------------------------------------------------;
	lda	row_counter	; get first char 
	cmp	#08h
	beq	Screen2Column
;-------------------------------------------------------------------------;
	;sta	current_char	; make it the current char (not needed)
;-------------------------------------------------------------------------;
DrawFlexPattern:
;-------------------------------------------------------------------------;
	;lda	current_char	; current char (already loaded)
	sta	REG_VMDATAL	; write it into V-Ram
	stz	REG_VMDATAH	; clear pallete #
	;lda	current_char	; (already loaded)
	clc
	adc	#08h		; add #8 to the current char value
				; since our grid will be 32 columns
				; and 8 rows we add #8 to the current
				; char value for the next character
				; store it back
	;sta	current_char	; (not needed)
	inx
	cpx	#0020h		; did we do 32 columns?
	bne	DrawFlexPattern
;-------------------------------------------------------------------------;
	inc	row_counter
	bra	DrawChar
;-------------------------------------------------------------------------;
	;ldx	#0000h		; set X back to $0	(changed start of drawchar
	;inc	row_counter	 increase Row counter       ...to save some bytes)
	;lda	row_counter
	;cmp	#08h		; did we do all 8 rows?
	;bne	drawchar	
;-------------------------------------------------------------------------;
Screen2Column:
;---------------------------------------;---------------------------------;
	lda	#VMAIN_INCH|VMAIN_INC32	;
	sta	REG_VMAIN		; inc on VMDATAH write and increase VMADDR 
	ldx	#BG1MAP+540h		; by 32 as we're dealing with only a
	stx	REG_VMADDL		; single column
	lda	#01h			;
	xba				;
	lda	#00h			;
;---------------------------------------;
:	sta	REG_VMDATAL		; store 00h-07h
	xba				;
	sta	REG_VMDATAH		; store 01h
	xba				;
	ina				;
	cmp	#08h			;
	bne	:-			;
;---------------------------------------;
	rts

; this is the basic idea of how a flex grid looks like:
; this grid is 5 columns * 5 rows
;  
; @EJOT
; AFKPU
; BGLQV
; CHMRW 
; DINSX


;=========================================================================;
ClearRam:
;=========================================================================;
	lda	#00h
	ldx	#0000h
;-------------------------------------------------------------------------;
:	sta	GFX_BUFFER,x	; clear Flex graphics buffer
	inx
	cpx	#0800h		; clear 2k of ram
	bne	:-
;-------------------------------------------------------------------------;
	lda	#20h
	ldx	#0021h
;-------------------------------------------------------------------------;
:	sta	RAM_SCROLLTEXT,x	; clear ram area where we will
	dex				; store scroll text data by filling
	bpl	:-			; with spaces
	rts


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
SINE:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
 .byte	032,032,033,034,035,035,036,037,038,038,039,040,041,041,042,043
 .byte	044,044,045,046,046,047,048,048,049,050,050,051,051,052,053,053
 .byte	054,054,055,055,056,056,057,057,058,058,059,059,059,060,060,060
 .byte	061,061,061,061,062,062,062,062,062,063,063,063,063,063,063,063
 .byte	063,063,063,063,063,063,063,063,062,062,062,062,062,061,061,061
 .byte	061,060,060,060,059,059,059,058,058,057,057,056,056,055,055,054
 .byte	054,053,053,052,051,051,050,050,049,048,048,047,046,046,045,044
 .byte	044,043,042,041,041,040,039,038,038,037,036,035,035,034,033,032
 .byte	032,031,030,029,028,028,027,026,025,025,024,023,022,022,021,020
 .byte	019,019,018,017,017,016,015,015,014,013,013,012,012,011,010,010
 .byte	009,009,008,008,007,007,006,006,005,005,004,004,004,003,003,003
 .byte	002,002,002,002,001,001,001,001,001,000,000,000,000,000,000,000
 .byte	000,000,000,000,000,000,000,000,001,001,001,001,001,002,002,002
 .byte	002,003,003,003,004,004,004,005,005,006,006,007,007,008,008,009
 .byte	009,010,010,011,012,012,013,013,014,015,015,016,017,017,018,019
 .byte	019,020,021,022,022,023,024,025,025,026,027,028,028,029,030,031
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
BG1_PAL:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.word	BLACK
	.word	DK_GRAY
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
BG2_PAL:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.word	BLACK
	.word	YELLOW
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
BG3_PAL:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.word	BLACK
	.word	RED
	.word	PURPLE
	.word	GREEN
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
LIST_TM:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.byte	$50,TM_BG4
	.byte	$40,TM_BG1|TM_BG2
	.byte	1,TM_BG3|TM_BG4
	.byte	0
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
SCROLLTEXT:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.byte	"                                "
	.byte	"HI THERE! HERE'S A FLEX SCROLLER FOR YOU "
	.byte	"TO PLAY WITH!   THIS WASN'T CODED TO RUN "
	.byte	"FAST, BUT JUST TO SEE IT IN ACTION. "
	.byte	" THE ENTIRE FLEX AND SCROLL ROUTINE TAKE UP "
	.byte	"ABOUT 3/4 OF THE RASTER TIME! THAT'S A LOT! "
	.byte	"BUT THE CODE COULD ALWAYS BE FIXED TO RUN FASTER.  "
	.byte	"THE SIZE OF THIS ENTIRE ROUTINE OUT-CLASSES THE C64 "
	.byte	"VERSION!   THE C64 WAS TOO SLOW AND YOU HAD TO "
	.byte	"CONTROL EACH COLUMN SEPARATLY...   "
	.byte	"WHO CARES!  THIS IS A FAST AND NICE JOB.. "
	.byte	" SEE YA LATER!    -PAN-/ANTHROX            "
	.byte	"                                ",0
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
TEXT:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.byte	"           FLEX FIXED           "
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
TEXT_END:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;

