;-------------------------------------------------------------------------;
.include "graphics.inc"
.include "snes.inc"
.include "snes_decompress.inc"
.include "snes_zvars.inc"
;-------------------------------------------------------------------------;
.export Mode5
;-------------------------------------------------------------------------;
BLANK_LINES = 32
TOP_LINES = 80
BOT_LINES = 80

BG1GFX	= 00000h
BG2GFX	= 03000h
BG1MAP	= 07000h
BG2MAP	= 07800h

RAM_INIDISP = 0600h
TMP_RAM = 7e2000h
;-------------------------------------------------------------------------;


;-------------------------------------------------------------------------;
fadem = m4
fade = m4+1
tilemap_high = m6
vram_max = m7
;-------------------------------------------------------------------------;


;/////////////////////////////////////////////////////////////////////////;
.code
;/////////////////////////////////////////////////////////////////////////;


;-------------------------------------------------------------------------;
	.a8
	.i16
;-------------------------------------------------------------------------;


;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::;
Mode5:
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::;
	rep	#10h
	sep	#20h

	DoDecompressDataVram gfx_mode5aTiles, BG1GFX*2
	DoDecompressDataVram gfx_mode5bTiles, BG2GFX*2
	DoDecompressDataVram gfx_mode5aMap, BG1MAP*2
	DoDecompressDataVram gfx_mode5bMap, BG2MAP*2
	DoCopyPalette gfx_mode5aPal, 0, 16
	DoCopyPalette gfx_mode5bPal, 16,16

	; snesgrit doesn't do mode 5 yet so we need to maniuplate the tilemaps
	; it has given us - due to lack of mode 5 support we cannot reduce
	; duplicate tiles so we'll run out of room quick!

	lda	#%00000
	ldx	#BG1MAP			; src & dest
	ldy	#0280h			; 80 pixels high
	jsr	ConvertToMode5Map
		;%vhopppcc
	lda	#%00000100		; palette 1
	ldx	#BG2MAP
	ldy	#0280h
	jsr	ConvertToMode5Map

	lda	#TM_BG1
	sta	REG_TM
	sta	REG_TS

	stz	fadem
	stz	fade

	rep	#30h

	lda	#LIST_INIDISP_END-LIST_INIDISP
	ldx	#LIST_INIDISP
	ldy	#RAM_INIDISP
	mvn	80h,^LIST_INIDISP

	sep	#20h

	lda	#DMAP_XFER_MODE_1
	sta	REG_DMAP1
	ina
	sta	REG_DMAP3

	stz	REG_DMAP0
	lda	#<REG_BGMODE
	sta	REG_BBAD0
	ldx	#LIST_BGMODE
	stx	REG_A1T0L
	stz	REG_A1B0

	lda	#<REG_BG1SC
	sta	REG_BBAD1
	ldx	#LIST_SC
	stx	REG_A1T1L
	stz	REG_A1B1

	stz	REG_DMAP2
	lda	#<REG_BG12NBA
	sta	REG_BBAD2
	ldx	#LIST_NBA
	stx	REG_A1T2L
	stz	REG_A1B2

	lda	#<REG_BG1VOFS
	sta	REG_BBAD3
	ldx	#LIST_BG1VOFS
	stx	REG_A1T3L
	stz	REG_A1B3

	stz	REG_DMAP4
	lda	#<REG_INIDISP
	sta	REG_BBAD4
	ldx	#RAM_INIDISP
	stx	REG_A1T4L
	stz	REG_A1B4

	lda	#%11111
	sta	REG_HDMAEN

	lda	#NMI_ON|NMI_JOYPAD
	sta	REG_NMITIMEN

	cli
;=========================================================================;
Loop:
;=========================================================================;
	lda	REG_RDNMI
	bpl	Loop

	jsr	Fade
	lda	fade
	bne	Loop

end:	wai
	bra	end



;=========================================================================;
Fade:
;=========================================================================;
	lda	fadem
	beq	FadeIn
;-------------------------------------------------------------------------;
	cmp	#2
	beq	FadeOut
	rts
;-------------------------------------------------------------------------;
FadeIn:
;-------------------------------------------------------------------------;
	inc	fade
	lda	fade
	cmp	#10h
	bcs	NextMode
;-------------------------------------------------------------------------;
dof:	sta	RAM_INIDISP+(FADE-LIST_INIDISP)
	sta	RAM_INIDISP+(FADE-LIST_INIDISP)+2
	rts
;-------------------------------------------------------------------------;
NextMode:
;-------------------------------------------------------------------------;
	inc	fadem
	rts
;-------------------------------------------------------------------------;
FadeOut:
;-------------------------------------------------------------------------;
	dec
	lda	fade
	bmi	NextMode
	bra	dof


;=========================================================================;
ConvertToMode5Map:
;=========================================================================;
	sty	vram_max
	stx	REG_VMADD
	sta	tilemap_high+1
	phx

	lda	#VMAIN_INCH
	sta	REG_VMAIN

	ldx	#0000h
	txy

	rep	#30h

	lda	REG_VMDATAREAD		; dummy read after this we'll be
;---------------------------------------; able to get the data we expect
:	lda	REG_VMDATAREAD
	asl
	and	#03ffh
	sta	TMP_RAM,x
	inx
	inx
	cpx	vram_max
	bne	:-
;-------------------------------------------------------------------------;
	plx
	stx	REG_VMADD
	tyx
;-------------------------------------------------------------------------;
:	lda	TMP_RAM,x
	ora	tilemap_high
	sta	REG_VMDATAL
	inx
	inx
	cpx	vram_max
	bne	:-
;-------------------------------------------------------------------------;
	lda	#0ah		; blank rest of screen
:	sta	REG_VMDATAL
	inx
	cpx	#0400h
	bne	:-
;-------------------------------------------------------------------------;
	sep	#20h

	lda	#00h
:	sta	TMP_RAM,x
	dex
	bpl	:-

	rts



;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
LIST_BGMODE:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.byte	BLANK_LINES+TOP_LINES,	BGMODE_5
	.byte	BOT_LINES,		BGMODE_5
	.byte	0
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
LIST_SC:			; SC1       SC2
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.byte	BLANK_LINES,	BG1MAP>>8,BG1MAP>>8
	.byte	TOP_LINES,	BG1MAP>>8,BG1MAP>>8
	.byte	BOT_LINES,	BG2MAP>>8,BG2MAP>>8
	.byte	1,		BG2MAP>>8,BG2MAP>>8
	.byte	0
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
LIST_NBA:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.byte	BLANK_LINES,	BG1GFX>>12+BG1GFX>>8
	.byte	TOP_LINES,	BG1GFX>>12+BG1GFX>>8
	.byte	BOT_LINES,	BG2GFX>>12+BG2GFX>>8
	.byte	1,		BG2GFX>>12+BG2GFX>>8
	.byte	0
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
LIST_BG1VOFS:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.byte	BLANK_LINES,	$ff,$00
	.byte	TOP_LINES,	$df,$00
	.byte	BOT_LINES,	$8f,$00
	.byte	1,		$ff,$00
	.byte	0
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;


;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
LIST_INIDISP:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
	.byte	BLANK_LINES,0
	.byte	TOP_LINES
FADE:	.byte	$00
	.byte	BOT_LINES,$00
	.byte	1,0
	.byte	0
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
LIST_INIDISP_END:
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=;
